@@ -1,6 +1,6 @@
 /* common.c -- Common functions.
  *
- * Copyright (C) 2013-2020 Artyom V. Poptsov <poptsov.artyom@gmail.com>
+ * Copyright (C) 2013-2021 Artyom V. Poptsov <poptsov.artyom@gmail.com>
  *
  * This file is part of Guile-SSH.
  *
@@ -27,9 +27,9 @@
 
 /* Convert the SSH constant VALUE to a Scheme symbol */
 SCM
-_ssh_const_to_scm (const struct symbol_mapping *types, int value)
+gssh_symbol_to_scm (const gssh_symbol_t *types, int value)
 {
-  const struct symbol_mapping *t;
+  const gssh_symbol_t *t;
   for (t = types; t->symbol; ++t)
     {
       if (t->value == value)
@@ -42,10 +42,10 @@ _ssh_const_to_scm (const struct symbol_mapping *types, int value)
 
    Return the apropriate structure that contains the needed
    constant. */
-const struct symbol_mapping *
-_scm_to_ssh_const (const struct symbol_mapping *types, SCM value)
+const gssh_symbol_t *
+gssh_symbol_from_scm (const gssh_symbol_t *types, SCM value)
 {
-  const struct symbol_mapping *t;
+  const gssh_symbol_t *t;
   char *sym = scm_to_locale_string (scm_symbol_to_string (value));
   for (t = types; t->symbol; ++t)
     {
@@ -64,4 +64,36 @@ _scm_object_hex_address (SCM obj)
   return scm_number_to_string (scm_object_address (obj), scm_from_uint (16U));
 }
 
+
+
+void
+set_smob_callbacks(scm_t_bits tag,
+                   gc_mark_callback_t   mark_cb,
+                   gc_free_callback_t   free_cb,
+                   gc_equalp_callback_t equalp_cb,
+                   gc_print_callback_t  print_cb)
+{
+    scm_set_smob_mark(tag, mark_cb);
+    scm_set_smob_free(tag, free_cb);
+    scm_set_smob_print(tag, print_cb);
+    scm_set_smob_equalp(tag, equalp_cb);
+}
+
+
+/** Procedure that converts an SCM object to an Guile-UDev structure
+ * pointer.
+ */
+SCM compare_objects(SCM x1, SCM x2, converter_t converter)
+{
+    void* d1 = converter(x1);
+    void* d2 = converter(x2);
+    if ((! d1) || (! d2)) {
+        return SCM_BOOL_F;
+    } else if (d1 != d2) {
+        return SCM_BOOL_F;
+    } else {
+        return SCM_BOOL_T;
+    }
+}
+
 /* common.c ends here. */
