@@ -38,7 +38,9 @@
 
 scm_t_bits key_tag; /* Smob tag. */
 
-static const struct symbol_mapping key_types[] = {
+static const char* GSSH_KEY_TYPE_NAME = "key";
+
+static const gssh_symbol_t key_types[] = {
   { "dss",     SSH_KEYTYPE_DSS     },
   { "rsa",     SSH_KEYTYPE_RSA     },
   { "rsa1",    SSH_KEYTYPE_RSA1    },
@@ -60,17 +62,17 @@ static const struct symbol_mapping key_types[] = {
 
 /* Smob marking */
 static SCM
-mark_key_smob (SCM key_smob)
+_mark (SCM key_smob)
 {
-  struct key_data *kd = _scm_to_key_data (key_smob);
+  gssh_key_t *kd = gssh_key_from_scm (key_smob);
   return kd->parent;
 }
 
 /* Free the smob. */
 static size_t
-free_key_smob (SCM arg1)
+_free (SCM arg1)
 {
-  struct key_data *data = (struct key_data *) SCM_SMOB_DATA (arg1);
+  gssh_key_t *data = (gssh_key_t *) SCM_SMOB_DATA (arg1);
 
   if (scm_is_false (data->parent))
     {
@@ -83,10 +85,16 @@ free_key_smob (SCM arg1)
   return 0;
 }
 
-static int
-print_key (SCM smob, SCM port, scm_print_state *pstate)
+static SCM
+_equalp (SCM x1, SCM x2)
 {
-  struct key_data *key_data = _scm_to_key_data (smob);
+  return compare_objects(x1, x2, (converter_t) gssh_key_from_scm);
+}
+
+static int
+_print (SCM smob, SCM port, scm_print_state *pstate)
+{
+  gssh_key_t *key_data = gssh_key_from_scm (smob);
   SCM type = guile_ssh_key_get_type (smob);
 
   scm_puts ("#<key ", port);
@@ -106,119 +114,31 @@ print_key (SCM smob, SCM port, scm_print_state *pstate)
 SCM
 _ssh_key_type_to_scm (int type)
 {
-  return _ssh_const_to_scm (key_types, type);
+  return gssh_symbol_to_scm (key_types, type);
 }
 
-const struct symbol_mapping *
+const gssh_symbol_t *
 _scm_to_ssh_key_type (SCM type)
 {
-  return _scm_to_ssh_const (key_types, type);
-}
-
-
-/* Get the type of the key KEY_SMOB.
-
-   Return a key type as a Scheme symbol.  The type can be one of the
-   following list: 'dss, 'rsa, 'rsa1, 'unknown */
-SCM_DEFINE (guile_ssh_key_get_type, "get-key-type", 1, 0, 0,
-            (SCM key),
-            "\
-Get a symbol that represents the type of the SSH key KEY.\n\
-Possible types are: 'dss, 'rsa, 'rsa1, 'ecdsa, 'unknown\
-")
-{
-  struct key_data *data = _scm_to_key_data (key);
-  enum ssh_keytypes_e type = ssh_key_type (data->ssh_key);
-  return _ssh_key_type_to_scm (type);
-}
-
-SCM_DEFINE (guile_ssh_make_keypair, "make-keypair", 2, 0, 0,
-            (SCM type, SCM length),
-            "\
-Generate a keypair of specified TYPE and LENGTH.  This may take some time.\
-Return newly generated private key.  Throw `guile-ssh-error' on error.\
-")
-#define FUNC_NAME s_guile_ssh_make_keypair
-{
-  ssh_key key = NULL;
-  const struct symbol_mapping *c_type = _scm_to_ssh_key_type (type);
-  int c_length;
-  int res;
-
-  SCM_ASSERT (scm_is_unsigned_integer (length, 9, UINT32_MAX), length,
-              SCM_ARG2, FUNC_NAME);
-
-  if (! c_type)
-    guile_ssh_error1 (FUNC_NAME, "Wrong key type", type);
-
-  c_length = scm_to_int (length);
-  res = ssh_pki_generate (c_type->value, c_length, &key);
-  if (res == SSH_ERROR)
-    {
-      guile_ssh_error1 (FUNC_NAME, "Could not generate key",
-                        scm_list_2 (type, length));
-    }
-
-  return _scm_from_ssh_key (key, SCM_BOOL_F);
-}
-#undef FUNC_NAME
-
-
-/* Predicates */
-
-SCM_DEFINE (guile_ssh_is_key_p, "key?", 1, 0, 0,
-            (SCM x),
-            "\
-Return #t if X is a SSH key, #f otherwise.\
-")
-{
-  return scm_from_bool (SCM_SMOB_PREDICATE (key_tag, x));
-}
-
-SCM_DEFINE (guile_ssh_is_public_key_p, "public-key?", 1, 0, 0,
-            (SCM x),
-            "\
-Return #t if X is a SSH key and it contains a public key, #f otherwise.\
-")
-{
-  return scm_from_bool (SCM_SMOB_PREDICATE (key_tag, x)
-                        && _public_key_p (_scm_to_key_data (x)));
-}
-
-SCM_DEFINE (guile_ssh_is_private_key_p, "private-key?", 1, 0, 0,
-            (SCM x),
-            "\
-Return #t if X is a SSH private-key, #f otherwise.\
-")
-{
-  return scm_from_bool (SCM_SMOB_PREDICATE (key_tag, x)
-                        && _private_key_p (_scm_to_key_data (x)));
-}
-
-static SCM
-equalp_key (SCM x1, SCM x2)
-{
-  struct key_data *key1 = _scm_to_key_data (x1);
-  struct key_data *key2 = _scm_to_key_data (x2);
-
-  if ((! key1) || (! key2))
-    return SCM_BOOL_F;
-  else if (key1 != key2)
-    return SCM_BOOL_F;
-  else
-    return SCM_BOOL_T;
+  return gssh_symbol_from_scm (key_types, type);
 }
 
 
 /* Helper procedures */
 
-SCM
-_scm_from_ssh_key (ssh_key key, SCM parent)
+gssh_key_t*
+make_gssh_key ()
 {
-  struct key_data *key_data;
+    return (gssh_key_t *) scm_gc_malloc (sizeof (gssh_key_t),
+                                         GSSH_KEY_TYPE_NAME);
+}
+
+SCM
+gssh_key_to_scm (ssh_key key, SCM parent)
+{
+  gssh_key_t *key_data;
   SCM key_smob;
-  key_data = (struct key_data *) scm_gc_malloc (sizeof (struct key_data),
-                                                "ssh key");
+  key_data = make_gssh_key ();
   key_data->ssh_key = key;
   key_data->parent = parent;
   SCM_NEWSMOB (key_smob, key_tag, key_data);
@@ -226,23 +146,23 @@ _scm_from_ssh_key (ssh_key key, SCM parent)
 }
 
 /* Convert X to a SSH key */
-struct key_data *
-_scm_to_key_data (SCM x)
+gssh_key_t *
+gssh_key_from_scm (SCM x)
 {
   scm_assert_smob_type (key_tag, x);
-  return (struct key_data *) SCM_SMOB_DATA (x);
+  return (gssh_key_t *) SCM_SMOB_DATA (x);
 }
 
 /* Check that KEY is a SSH private key. */
 int
-_private_key_p (struct key_data *key)
+_private_key_p (gssh_key_t *key)
 {
   return ssh_key_is_private (key->ssh_key);
 }
 
 /* Check that KEY is a SSH public key */
 int
-_public_key_p (struct key_data *key)
+_public_key_p (gssh_key_t *key)
 {
   return ssh_key_is_public (key->ssh_key);
 }
@@ -252,11 +172,8 @@ _public_key_p (struct key_data *key)
 void
 init_key_type (void)
 {
-  key_tag = scm_make_smob_type ("key", sizeof (struct key_data));
-  scm_set_smob_mark (key_tag, mark_key_smob);
-  scm_set_smob_free (key_tag, free_key_smob);
-  scm_set_smob_print (key_tag, print_key);
-  scm_set_smob_equalp (key_tag, equalp_key);
+  key_tag = scm_make_smob_type (GSSH_KEY_TYPE_NAME, sizeof (gssh_key_t));
+  set_smob_callbacks (key_tag, _mark, _free, _equalp, _print);
 
 #include "key-type.x"
 }
