@@ -46,7 +46,7 @@ Return value is undefined.\
 ")
 #define FUNC_NAME s_guile_ssh_message_reply_default
 {
-  struct message_data *msg_data = _scm_to_message_data (msg);
+  gssh_message_t* msg_data = gssh_message_from_scm (msg);
   int res = ssh_message_reply_default (msg_data->message);
   _gssh_log_debug_format(FUNC_NAME, msg, "result: %d", res);
   if (res != SSH_OK)
@@ -64,7 +64,7 @@ Return value is undefined.\
 ")
 #define FUNC_NAME s_guile_ssh_message_service_reply_success
 {
-  struct message_data *msg_data = _scm_to_message_data (msg);
+  gssh_message_t* msg_data = gssh_message_from_scm (msg);
   int res = ssh_message_service_reply_success (msg_data->message);
   _gssh_log_debug_format(FUNC_NAME, msg, "result: %d", res);
   if (res != SSH_OK)
@@ -82,7 +82,7 @@ Return value is undefined.\
 ")
 #define FUNC_NAME s_guile_ssh_message_auth_reply_success
 {
-  struct message_data *msg_data = _scm_to_message_data (msg);
+  gssh_message_t* msg_data = gssh_message_from_scm (msg);
   int c_partial_p = scm_to_bool (partial_p);
   int res = ssh_message_auth_reply_success (msg_data->message, c_partial_p);
   _gssh_log_debug_format(FUNC_NAME, msg, "result: %d", res);
@@ -104,7 +104,7 @@ Return value is undefined.\
 ")
 #define FUNC_NAME s_guile_ssh_message_auth_reply_public_key_ok
 {
-  struct message_data *msg_data = _scm_to_message_data (msg);
+  gssh_message_t* msg_data = gssh_message_from_scm (msg);
   int res = ssh_message_auth_reply_pk_ok_simple (msg_data->message);
   _gssh_log_debug_format(FUNC_NAME, msg, "result: %d", res);
   if (res != SSH_OK)
@@ -122,7 +122,7 @@ Return value is undefined.\
 ")
 #define FUNC_NAME s_guile_ssh_message_channel_request_reply_success
 {
-  struct message_data *msg_data = _scm_to_message_data (msg);
+  gssh_message_t* msg_data = gssh_message_from_scm (msg);
   int res = ssh_message_channel_request_reply_success (msg_data->message);
   _gssh_log_debug_format(FUNC_NAME, msg, "result: %d", res);
   if (res != SSH_OK)
@@ -139,15 +139,15 @@ Accept open-channel request.\n\
 Return a new SSH channel.\
 ")
 {
-  struct message_data *msg_data = _scm_to_message_data (msg);
+  gssh_message_t* msg_data = gssh_message_from_scm (msg);
   ssh_channel ch;
 
   ch = ssh_message_channel_request_open_reply_accept (msg_data->message);
   if (! ch)
     return SCM_BOOL_F;
 
-  SCM channel = _scm_from_channel_data (ch, msg_data->session,
-                                        SCM_RDNG | SCM_WRTNG);
+  SCM channel = ssh_channel_to_scm (ch, msg_data->session,
+                                    SCM_RDNG | SCM_WRTNG);
 
   SCM_SET_CELL_TYPE (channel, SCM_CELL_TYPE (channel) | SCM_OPN);
 
@@ -159,7 +159,7 @@ SCM_DEFINE (gssh_message_global_request_reply_success,
             (SCM msg, SCM bound_port), "")
 #define FUNC_NAME s_gssh_message_global_request_reply_success
 {
-  struct message_data *md = _scm_to_message_data (msg);
+  gssh_message_t* md = gssh_message_from_scm (msg);
   int res;
 
   SCM_ASSERT (scm_is_unsigned_integer (bound_port, 0, UINT16_MAX), bound_port,
@@ -181,7 +181,7 @@ SCM_DEFINE (gssh_message_global_request_reply_success,
 
 
 
-static struct symbol_mapping req_types[] = {
+static gssh_symbol_t req_types[] = {
   { "request-auth",         SSH_REQUEST_AUTH         },
   { "request-channel-open", SSH_REQUEST_CHANNEL_OPEN },
   { "request-channel",      SSH_REQUEST_CHANNEL      },
@@ -190,7 +190,7 @@ static struct symbol_mapping req_types[] = {
   { NULL,           -1                               }
 };
 
-static struct symbol_mapping req_auth_subtypes[] = {
+static gssh_symbol_t req_auth_subtypes[] = {
   { "auth-method-unknown",     SSH_AUTH_METHOD_UNKNOWN     },
   { "auth-method-none",        SSH_AUTH_METHOD_NONE        },
   { "auth-method-password",    SSH_AUTH_METHOD_PASSWORD    },
@@ -200,7 +200,7 @@ static struct symbol_mapping req_auth_subtypes[] = {
   { NULL,                      -1                          }
 };
 
-static struct symbol_mapping req_channel_subtypes[] = {
+static gssh_symbol_t req_channel_subtypes[] = {
   { "channel-request-unknown",       SSH_CHANNEL_REQUEST_UNKNOWN       },
   { "channel-request-pty",           SSH_CHANNEL_REQUEST_PTY           },
   { "channel-request-exec",          SSH_CHANNEL_REQUEST_EXEC          },
@@ -211,7 +211,7 @@ static struct symbol_mapping req_channel_subtypes[] = {
   { NULL,                            -1                                }
 };
 
-static struct symbol_mapping req_channel_open_subtypes[] = {
+static gssh_symbol_t req_channel_open_subtypes[] = {
   { "channel-unknown",         SSH_CHANNEL_UNKNOWN         },
   { "channel-session",         SSH_CHANNEL_SESSION         },
   { "channel-direct-tcpip",    SSH_CHANNEL_DIRECT_TCPIP    },
@@ -220,14 +220,14 @@ static struct symbol_mapping req_channel_open_subtypes[] = {
   { NULL,                      -1                          }
 };
 
-static struct symbol_mapping req_global_subtypes[] = {
+static gssh_symbol_t req_global_subtypes[] = {
   { "global-request-unknown",              SSH_GLOBAL_REQUEST_UNKNOWN              },
   { "global-request-tcpip-forward",        SSH_GLOBAL_REQUEST_TCPIP_FORWARD        },
   { "global-request-cancel-tcpip-forward", SSH_GLOBAL_REQUEST_CANCEL_TCPIP_FORWARD },
   { NULL,                                  -1                                      }
 };
 
-static struct symbol_mapping pubkey_state_type[] = {
+static gssh_symbol_t pubkey_state_type[] = {
   { "error", SSH_PUBLICKEY_STATE_ERROR },
   { "none",  SSH_PUBLICKEY_STATE_NONE  },
   { "valid", SSH_PUBLICKEY_STATE_VALID },
@@ -244,25 +244,25 @@ _ssh_message_type_to_scm (ssh_message msg)
 {
   int type     = ssh_message_type (msg);
   int subtype  = ssh_message_subtype (msg);
-  SCM scm_type = _ssh_const_to_scm (req_types, type);
+  SCM scm_type = gssh_symbol_to_scm (req_types, type);
   SCM scm_subtype;
 
   switch (type)
     {
     case SSH_REQUEST_AUTH:
-      scm_subtype = _ssh_const_to_scm (req_auth_subtypes, subtype);
+      scm_subtype = gssh_symbol_to_scm (req_auth_subtypes, subtype);
       return scm_list_2 (scm_type, scm_subtype);
 
     case SSH_REQUEST_CHANNEL_OPEN:
-      scm_subtype = _ssh_const_to_scm (req_channel_open_subtypes, subtype);
+      scm_subtype = gssh_symbol_to_scm (req_channel_open_subtypes, subtype);
       return scm_list_2 (scm_type, scm_subtype);
 
     case SSH_REQUEST_CHANNEL:
-      scm_subtype = _ssh_const_to_scm (req_channel_subtypes, subtype);
+      scm_subtype = gssh_symbol_to_scm (req_channel_subtypes, subtype);
       return scm_list_2 (scm_type, scm_subtype);
 
     case SSH_REQUEST_GLOBAL:
-      scm_subtype = _ssh_const_to_scm (req_global_subtypes, subtype);
+      scm_subtype = gssh_symbol_to_scm (req_global_subtypes, subtype);
       return scm_list_2 (scm_type, scm_subtype);
 
     case SSH_REQUEST_SERVICE:
@@ -281,7 +281,7 @@ SCM_DEFINE (guile_ssh_message_get_type,
 Get type of the message MSG.\
 ")
 {
-  struct message_data *message_data = _scm_to_message_data (msg);
+  gssh_message_t* message_data = gssh_message_from_scm (msg);
   return _ssh_message_type_to_scm (message_data->message);
 }
 
@@ -309,9 +309,9 @@ get_auth_req (ssh_message msg, SCM scm_msg) /* FIXME: accept only SCM */
   else
     SCM_SIMPLE_VECTOR_SET (result, 1, SCM_BOOL_F);
 
-  SCM_SIMPLE_VECTOR_SET (result, 2, _scm_from_ssh_key (public_key, scm_msg));
+  SCM_SIMPLE_VECTOR_SET (result, 2, gssh_key_to_scm(public_key, scm_msg));
 
-  pkey_state = _ssh_const_to_scm (pubkey_state_type,
+  pkey_state = gssh_symbol_to_scm (pubkey_state_type,
                                   (int) ssh_message_auth_publickey_state (msg));
   SCM_SIMPLE_VECTOR_SET (result, 3, pkey_state);
 
@@ -436,7 +436,7 @@ Get a request object from the message MSG\
 ")
 #define FUNC_NAME s_guile_ssh_message_get_req
 {
-  struct message_data *message_data = _scm_to_message_data (msg);
+  gssh_message_t* message_data = gssh_message_from_scm (msg);
   ssh_message ssh_msg = message_data->message;
   int type = ssh_message_type (ssh_msg);
 
@@ -485,7 +485,7 @@ Get a request object from the message MSG\
 
     default:
       guile_ssh_error1 (FUNC_NAME, "Wrong message type",
-                        _ssh_const_to_scm (req_types, type));
+                        gssh_symbol_to_scm (req_types, type));
 
     }
 
@@ -511,7 +511,7 @@ Return value is undefined.\
 ")
 #define FUNC_NAME s_guile_ssh_message_auth_set_methods_x
 {
-  struct message_data *message_data = _scm_to_message_data (msg);
+  gssh_message_t* message_data = gssh_message_from_scm (msg);
   int methods = 0;
   int res;
 
@@ -550,7 +550,7 @@ SCM_DEFINE (guile_ssh_message_get_session,
 Get the session from which the MESSAGE was received.  Return the session.\
 ")
 {
-  struct message_data *md = _scm_to_message_data (message);
+  gssh_message_t* md = gssh_message_from_scm (message);
   return md->session;
 }
 
