@@ -33,12 +33,6 @@
 #include "message-type.h"
 #include "log.h"
 
-/* SSH option mapping. */
-struct option {
-  char* symbol;
-  int   type;
-};
-
 /* Guile SSH specific options that are aimed to unificate the way of session
    configuration. */
 enum gssh_session_options {
@@ -49,7 +43,7 @@ enum gssh_session_options {
 
 /* SSH options mapping to Guile symbols. */
 
-static struct symbol_mapping session_options[] = {
+static gssh_symbol_t session_options[] = {
   { "host",               SSH_OPTIONS_HOST               },
   { "port",               SSH_OPTIONS_PORT               },
   { "fd",                 SSH_OPTIONS_FD                 },
@@ -71,13 +65,55 @@ static struct symbol_mapping session_options[] = {
   { "stricthostkeycheck", SSH_OPTIONS_STRICTHOSTKEYCHECK },
   { "compression",        SSH_OPTIONS_COMPRESSION        },
   { "compression-level",  SSH_OPTIONS_COMPRESSION_LEVEL  },
+
 #if HAVE_LIBSSH_0_8_1
   { "nodelay",            SSH_OPTIONS_NODELAY            },
 #endif
+
+#if HAVE_LIBSSH_0_8_3
+  /* Preferred public key algorithms to be used for authentication
+     (comma-separated list as a string). Example:
+     "ssh-rsa,rsa-sha2-256,ssh-dss,ecdh-sha2-nistp256"
+
+     This option was added to the libssh in
+     4521ab73b6858efa0083ac96a1775719b1f649ae */
+  { "public-key-accepted-types", SSH_OPTIONS_PUBLICKEY_ACCEPTED_TYPES },
+#endif
+
   { "callbacks",          GSSH_OPTIONS_CALLBACKS         },
   { NULL,                 -1 }
 };
 
+
+/* Create a new session. */
+SCM_DEFINE (guile_ssh_make_session, "%make-session", 0, 0, 0,
+            (),
+            "\
+Create a new session.\
+")
+{
+  gssh_session_t *session_data = make_gssh_session ();
+
+  session_data->ssh_session = ssh_new ();
+  if (session_data->ssh_session == NULL)
+    return SCM_BOOL_F;
+  session_data->callbacks = SCM_BOOL_F;
+  session_data->channels  = SCM_EOL;
+
+  return gssh_session_to_scm (session_data);
+}
+
+
+/* Predicates */
+SCM_DEFINE (guile_ssh_is_session_p, "session?", 1, 0, 0,
+            (SCM x),
+            "\
+Return #t if X is a SSH session, #f otherwise.\
+")
+{
+  return scm_from_bool (SCM_SMOB_PREDICATE (session_tag, x));
+}
+
 /* Blocking flush of the outgoing buffer.
 
    Return on of the following symbols: 'ok, 'error. 'again.
@@ -93,7 +129,7 @@ Return on of the following symbols: 'ok, 'error, 'again.\
 ")
 #define FUNC_NAME s_guile_ssh_blocking_flush
 {
-  struct session_data *data = _scm_to_session_data (session_smob);
+  gssh_session_t *data = gssh_session_from_scm (session_smob);
 
   int c_timeout;                /* Timeout */
   int res;                      /* Result of a function call. */
@@ -104,6 +140,10 @@ Return on of the following symbols: 'ok, 'error, 'again.\
   c_timeout = scm_to_int (timeout);
 
   res = ssh_blocking_flush (data->ssh_session, c_timeout);
+  _gssh_log_debug_format (FUNC_NAME,
+                          scm_list_2 (session_smob, timeout),
+                          "result: %d",
+                          res);
   switch (res)
     {
     case SSH_OK:
@@ -116,6 +156,10 @@ Return on of the following symbols: 'ok, 'error, 'again.\
       return scm_from_locale_symbol ("error");
 
     default:                    /* Must not happen. */
+      _gssh_log_error_format (FUNC_NAME,
+                              scm_list_2 (session_smob, timeout),
+                              "Unknown result: %d",
+                              res);
       assert (0);
       return SCM_BOOL_F;
     }
@@ -210,9 +254,9 @@ set_port_opt (ssh_session session, int type, SCM value)
 /* Convert Scheme symbol to libssh constant and set the corresponding
    option to the value of the constant. */
 static inline int
-set_sym_opt (ssh_session session, int type, struct symbol_mapping *sm, SCM value)
+set_sym_opt (ssh_session session, int type, gssh_symbol_t *sm, SCM value)
 {
-  const struct symbol_mapping *opt = _scm_to_ssh_const (sm, value);
+  const gssh_symbol_t *opt = gssh_symbol_from_scm (sm, value);
   if (! opt)
     guile_ssh_error1 ("session-set!", "Wrong value", value);
   return ssh_options_set (session, type, &opt->value);
@@ -231,7 +275,7 @@ callback_set_p (SCM callbacks, const char* name)
 
 /* Get an element NAME of the callbacks alist from a session data SD. */
 static inline SCM
-callbacks_ref (const struct session_data *sd, const char* name)
+callbacks_ref (const gssh_session_t *sd, const char* name)
 {
   return scm_assoc_ref (sd->callbacks, scm_from_locale_symbol (name));
 }
@@ -244,11 +288,11 @@ libssh_global_request_callback (ssh_session session, ssh_message message,
                                 void *userdata)
 {
   SCM scm_session = (SCM) userdata;
-  struct session_data *sd = _scm_to_session_data (scm_session);
+  gssh_session_t *sd = gssh_session_from_scm (scm_session);
 
   SCM scm_callback = callbacks_ref (sd, "global-request-callback");
   SCM scm_userdata = callbacks_ref (sd, "user-data");
-  SCM scm_message = _scm_from_ssh_message (message, scm_session);
+  SCM scm_message = ssh_message_to_scm (message, scm_session);
 
   scm_call_3 (scm_callback, scm_session, scm_message, scm_userdata);
 }
@@ -260,7 +304,7 @@ static void
 libssh_connect_status_callback (void *userdata, float status)
 {
   SCM scm_session = (SCM) userdata;
-  struct session_data *sd = _scm_to_session_data (scm_session);
+  gssh_session_t *sd = gssh_session_from_scm (scm_session);
 
   SCM scm_callback = callbacks_ref (sd, "connect-status-callback");
   SCM scm_userdata = callbacks_ref (sd, "user-data");
@@ -278,7 +322,7 @@ scm_is_procedure (SCM x)
 
 /* Validate callback NAME.  Throw 'guile-ssh-error' exception on an error. */
 static void
-validate_callback (SCM session, const struct session_data *sd, const char* name)
+validate_callback (SCM session, const gssh_session_t *sd, const char* name)
 {
   if (! scm_is_procedure (callbacks_ref (sd, name)))
     {
@@ -297,7 +341,7 @@ validate_callback (SCM session, const struct session_data *sd, const char* name)
 
    Return SSH_OK if callbacks were set succesfully, SSH_ERROR otherwise. */
 static int
-set_callbacks (SCM session, struct session_data *sd, SCM callbacks)
+set_callbacks (SCM session, gssh_session_t *sd, SCM callbacks)
 {
   struct ssh_callbacks_struct *cb
     = (struct ssh_callbacks_struct *)
@@ -333,7 +377,7 @@ set_callbacks (SCM session, struct session_data *sd, SCM callbacks)
 
 /* Set an SSH session option. */
 static int
-set_option (SCM scm_session, struct session_data* sd, int type, SCM value)
+set_option (SCM scm_session, gssh_session_t* sd, int type, SCM value)
 {
   ssh_session session = sd->ssh_session;
 
@@ -354,6 +398,9 @@ set_option (SCM scm_session, struct session_data* sd, int type, SCM value)
     case SSH_OPTIONS_COMPRESSION_C_S:
     case SSH_OPTIONS_COMPRESSION_S_C:
     case SSH_OPTIONS_PROXYCOMMAND:
+#if HAVE_LIBSSH_0_8_3
+    case SSH_OPTIONS_PUBLICKEY_ACCEPTED_TYPES:
+#endif
       return set_string_opt (session, type, value);
 
     case SSH_OPTIONS_LOG_VERBOSITY:
@@ -380,6 +427,15 @@ set_option (SCM scm_session, struct session_data* sd, int type, SCM value)
     case GSSH_OPTIONS_CALLBACKS:
       return set_callbacks (scm_session, sd, value);
 
+#if ! HAVE_LIBSSH_0_8_3
+    case SSH_OPTIONS_PUBLICKEY_ACCEPTED_TYPES:
+        guile_ssh_error1 ("session-set!",
+                          "Option 'public-key-accepted-types' is not available"
+                          " in the current version of libssh (%s)",
+                          ssh_version (0));
+        break;
+#endif
+
     default:
       guile_ssh_error1 ("session-set!",
                         "Operation is not supported yet: %a~%",
@@ -398,13 +454,13 @@ Return value is undefined.\
 ")
 #define FUNC_NAME s_guile_ssh_session_set
 {
-  struct session_data* data = _scm_to_session_data (session);
-  const struct symbol_mapping *opt;	/* Session option */
+  gssh_session_t* data = gssh_session_from_scm (session);
+  const gssh_symbol_t *opt;	/* Session option */
   int res;                              /* Result of a function call */
 
   SCM_ASSERT (scm_is_symbol (option), option, SCM_ARG2, FUNC_NAME);
 
-  opt = _scm_to_ssh_const (session_options, option);
+  opt = gssh_symbol_from_scm (session_options, option);
 
   if(! opt)
     guile_ssh_error1 (FUNC_NAME, "No such option", option);
@@ -427,7 +483,7 @@ Return value is undefined.\
 
 
 /* Options whose values can be requested through `session-get' */
-static struct symbol_mapping session_options_getable[] = {
+static gssh_symbol_t session_options_getable[] = {
   { "host",         SSH_OPTIONS_HOST         },
   { "port",         SSH_OPTIONS_PORT         },
   { "user",         SSH_OPTIONS_USER         },
@@ -444,14 +500,14 @@ Get value of the OPTION.  Throw `guile-ssh-error' on an error.\
 ")
 #define FUNC_NAME s_guile_ssh_session_get
 {
-  struct session_data*sd     = _scm_to_session_data (session);
-  const struct symbol_mapping *opt = NULL;
+  gssh_session_t* sd       = gssh_session_from_scm (session);
+  const gssh_symbol_t *opt = NULL;
   SCM value = SCM_UNDEFINED;                    /*Value of the option */
   int res = SSH_OK;
 
   SCM_ASSERT (scm_is_symbol (option), option, SCM_ARG2, FUNC_NAME);
 
-  opt = _scm_to_ssh_const (session_options_getable, option);
+  opt = gssh_symbol_from_scm (session_options_getable, option);
   if (! opt)
     guile_ssh_error1 (FUNC_NAME, "Wrong option", option);
 
@@ -492,7 +548,7 @@ SCM_GSSH_DEFINE (gssh_session_parse_config, "%gssh-session-parse-config!", 2,
                  (SCM session, SCM file_name))
 #define FUNC_NAME s_gssh_session_parse_config
 {
-  struct session_data *sd = _scm_to_session_data (session);
+  gssh_session_t *sd = gssh_session_from_scm (session);
   int res;
   char* c_file_name = NULL;
 
@@ -543,8 +599,9 @@ Return one of the following symbols: 'ok, 'again, 'error\
 ")
 #define FUNC_NAME s_guile_ssh_connect_x
 {
-  struct session_data* data = _scm_to_session_data (session);
+  gssh_session_t* data = gssh_session_from_scm (session);
   int res = ssh_connect (data->ssh_session);
+  _gssh_log_debug_format (FUNC_NAME, session, "result: %d", res);
   switch (res)
     {
     case SSH_OK:
@@ -569,11 +626,23 @@ SCM_DEFINE (guile_ssh_disconnect, "disconnect!", 1, 0, 0,
 Disconnect from a session (client or server).\n\
 Return value is undefined.\
 ")
+#define FUNC_NAME s_guile_ssh_disconnect
 {
-  struct session_data* session_data = _scm_to_session_data (arg1);
-  ssh_disconnect (session_data->ssh_session);
+  gssh_session_t* session_data = gssh_session_from_scm (arg1);
+
+  _gssh_log_debug (FUNC_NAME, "Disconnecting session ...",
+                   arg1);
+  if (ssh_is_connected (session_data->ssh_session))
+    {
+      _gssh_log_debug (FUNC_NAME, "Closing channels", arg1);
+      gssh_session_close_all_channels_x (session_data);
+      ssh_disconnect (session_data->ssh_session);
+    }
+  _gssh_log_debug (FUNC_NAME, "Disconnecting session ... done",
+                   arg1);
   return SCM_UNDEFINED;
 }
+#undef FUNC_NAME
 
 SCM_DEFINE (guile_ssh_get_protocol_version, "get-protocol-version", 1, 0, 0,
             (SCM arg1),
@@ -583,7 +652,7 @@ Return 1 for SSH1, 2 for SSH2 or #f on error.\
 ")
 #define FUNC_NAME s_guile_ssh_get_protocol_version
 {
-  struct session_data* data = _scm_to_session_data (arg1);
+  gssh_session_t* data = gssh_session_from_scm (arg1);
   SCM ret;
   int version;
 
@@ -606,7 +675,7 @@ SCM_DEFINE (guile_ssh_get_error, "get-error", 1, 0, 1,
 Retrieve the error text message from the last error.\
 ")
 {
-  struct session_data* data = _scm_to_session_data (arg1);
+  gssh_session_t* data = gssh_session_from_scm (arg1);
   SCM error = scm_from_locale_string (ssh_get_error (data->ssh_session));
   return error;
 }
@@ -628,7 +697,7 @@ Return one of the following symbols: 'ok, 'known-changed, 'found-other,\n\
 ")
 #define FUNC_NAME s_guile_ssh_authenticate_server
 {
-  struct session_data* data = _scm_to_session_data (session);
+  gssh_session_t* data = gssh_session_from_scm (session);
   int res;
 
   GSSH_VALIDATE_CONNECTED_SESSION (data, session, SCM_ARG1);
@@ -639,6 +708,8 @@ Return one of the following symbols: 'ok, 'known-changed, 'found-other,\n\
   res = ssh_is_server_known (data->ssh_session);
 #endif
 
+  _gssh_log_debug_format (FUNC_NAME, session, "result: %d", res);
+
   switch (res)
     {
     case SSH_SERVER_KNOWN_OK:
@@ -674,14 +745,14 @@ Return server's public key.  Throw `guile-ssh-error' on error.\
 ")
 #define FUNC_NAME s_guile_ssh_get_server_public_key
 {
-  struct session_data *sd = _scm_to_session_data (session);
-  struct key_data *kd;
+  gssh_session_t *sd = gssh_session_from_scm (session);
+  gssh_key_t *kd;
   int res;
   SCM key_smob;
 
   GSSH_VALIDATE_CONNECTED_SESSION (sd, session, SCM_ARG1);
 
-  kd = (struct key_data *) scm_gc_malloc (sizeof (struct key_data), "ssh key");
+  kd = make_gssh_key ();
   /* TODO: Check `kd' for NULL. */
 
 #if HAVE_LIBSSH_0_8
@@ -690,6 +761,8 @@ Return server's public key.  Throw `guile-ssh-error' on error.\
   res = ssh_get_publickey (sd->ssh_session, &kd->ssh_key);
 #endif
 
+  _gssh_log_debug_format (FUNC_NAME, session, "result: %d", res);
+
   if (res != SSH_OK)
     guile_ssh_error1 (FUNC_NAME, "Unable to get the server key", session);
 
@@ -707,7 +780,7 @@ Return value is undefined.\
 ")
 #define FUNC_NAME s_guile_ssh_write_known_host
 {
-  struct session_data *session_data = _scm_to_session_data (session);
+  gssh_session_t *session_data = gssh_session_from_scm (session);
   int res;
 
   GSSH_VALIDATE_CONNECTED_SESSION (session_data, session, SCM_ARG1);
@@ -718,6 +791,8 @@ Return value is undefined.\
   res = ssh_write_knownhost (session_data->ssh_session);
 #endif
 
+  _gssh_log_debug_format (FUNC_NAME, session, "result: %d", res);
+
   if (res != SSH_OK)
     guile_ssh_session_error1 (FUNC_NAME, session_data->ssh_session, session);
 
@@ -734,11 +809,14 @@ SCM_DEFINE (guile_ssh_is_connected_p, "connected?", 1, 0, 0,
 Check if we are connected.\n\
 Return #f if we are connected to a server, #f if we aren't.\
 ")
+#define FUNC_NAME s_guile_ssh_is_connected_p
 {
-  struct session_data* data = _scm_to_session_data (arg1);
+  gssh_session_t* data = gssh_session_from_scm (arg1);
   int res = ssh_is_connected (data->ssh_session);
+  _gssh_log_debug_format (FUNC_NAME, arg1, "result: %d", res);
   return scm_from_bool (res);
 }
+#undef FUNC_NAME
 
 
 /* Initialize session related functions. */
