@@ -23,21 +23,36 @@
 #include <libssh/callbacks.h>
 #include "threads.h"
 
+/* Since version 0.8.0, when libssh is dynamically linked, it is not necessary
+   to call 'ssh_init'/'ssh_finalize' on systems which are fully supported with
+   regards to threading (that is, system with pthreads available).
+
+   See <https://api.libssh.org/stable/group__libssh.html> for details. */
+
+#if ! HAVE_LIBSSH_0_8
+
 /* Current SSH threading state */
 static int pthreads_state = SSH_PTHREADS_DISABLED;
 
+/* This procedure is called when the Guile-SSH library is unloaded. */
+__attribute__((__destructor__)) void
+cleanup ()
+{
+    ssh_finalize ();
+}
+
 /* Initialize threading if it has not been initialized yet. */
 void
 init_pthreads (void)
 {
-  if (pthreads_state == SSH_PTHREADS_DISABLED)
-    {
-#if ! HAVE_LIBSSH_0_8
-      ssh_threads_set_callbacks (ssh_threads_get_pthread ());
-#endif
-      ssh_init ();
-      pthreads_state = SSH_PTHREADS_ENABLED;
-    }
+    if (pthreads_state == SSH_PTHREADS_DISABLED)
+        {
+            ssh_threads_set_callbacks (ssh_threads_get_pthread ());
+            ssh_init ();
+            pthreads_state = SSH_PTHREADS_ENABLED;
+        }
 }
 
+#endif /* if ! HAVE_LIBSSH_0_8 */
+
 /* threads.c ends here */
