@@ -12,21 +12,29 @@
              (ssh log)
              (ssh server))
 
-(define (log message)
-  (let ((f (open-file "/tmp/b.txt" "a+")))
-    (display message f)
-    (newline f)
-    (close f)))
-
+
 (define (main args)
-  (log args)
   (let ((test-suite-name (list-ref args 1))
         (test-name       (list-ref args 2)))
+
+    (define (log message)
+      (let ((port (open-file (string-append test-suite-name "/log.txt") "a+")))
+        (format port
+                "~s: ~a~%"
+                (strftime "%Y-%m-%d %H:%M:%S" (localtime (current-time)))
+                message)
+        (close port)))
+
     (unless (file-exists? test-suite-name)
       (mkdir test-suite-name))
+
+    (log args)
+
     (set-log-userdata! test-name)
-    (setup-test-suite-logging! (string-append test-suite-name "/" test-name))
-    (log "test 1")
+    (setup-test-suite-logging! (format #f
+                                       "~a/~a-server"
+                                       test-suite-name
+                                       (sanitize-string test-name)))
     (let* ((port      (string->number (list-ref args 3)))
            (handler   (eval-string (list-ref args 4)))
            (s         (make-server
@@ -35,10 +43,12 @@
                        #:rsakey   %rsakey
                        #:dsakey   %dsakey
                        #:log-verbosity 'functions)))
-      (log "test 2")
       (server-listen s)
-      (let ((p (open-output-file (string-append test-name ".run"))))
+      (let ((p (open-output-file (format #f
+                                         "~a/~a-server.run"
+                                         test-suite-name
+                                         (sanitize-string test-name)))))
+        (format p "~a~%" (getpid))
         (close p))
-      (log handler)
-      (handler s)   (log "test 4"))))
-
+      (usleep 100)
+      (handler s))))
