@@ -91,16 +91,15 @@
    (lambda ()
      (call-with-connected-session/shell
       (lambda (session)
-        (let* ((n1   (make-node session))
-               (n2   (make-node session))
+        (let* ((node (make-node session))
                (data '())
                (proc '(lambda (n) (1+ n)))
-               (j1   (make-job 'map n1 data proc))
-               (j2   (set-job-node j1 n2)))
+               (j1   (make-job 'map #f data proc))
+               (j2   (set-job-node j1 node)))
           (and (not (eq? j1 j2))
                (eq? (job-type j1) (job-type j2))
-               (eq? (job-node j1) n1)
-               (eq? (job-node j2) n2)
+               (eq? (job-node j1) #f)
+               (eq? (job-node j2) node)
                (eq? (job-data j1) (job-data j2))
                (eq? (job-proc j1) (job-proc j2)))))))))
 
@@ -182,10 +181,14 @@ Unbound variable: e"
                    "socket:9:7: In procedure module-lookup: Unbound variable: e"))
    rrepl-get-result))
 
-(test-error-with-log/= "rrepl-get-result, unknown # object error"
-  'node-repl-error "Reader error: scm_lreadr: #<unknown port>:1:3: \
-Unknown # object: (#\\<): scheme@(guile-user)> \
-$4 = #<session #<undefined>@#<undefined>:22 (disconnected) 453fff>"
+;; Here we have to use regexps to match the error message because of
+;; differences between Guile 3.0.7 and older versions.
+;;
+;; See <https://github.com/artyom-poptsov/guile-ssh/issues/28>
+(test-error-with-log/match "rrepl-get-result, unknown # object error"
+  'node-repl-error "Reader error: .+: #<unknown port>:1:3: \
+Unknown # object: \\(.+\\): scheme@\\(guile-user\\)> \
+\\$4 = #<session #<undefined>@#<undefined>:22 \\(disconnected\\) 453fff>"
   (call-with-input-string
    (string-append  "scheme@(guile-user)> $4 = "
                    "#<session #<undefined>@#<undefined>:22 (disconnected) 453fff>")
@@ -246,7 +249,8 @@ $4 = #<session #<undefined>@#<undefined>:22 (disconnected) 453fff>"
 ;; The client uses distributed form 'with-ssh' to evaluate (+ 21 21).  The
 ;; server pretends to be a RREPL server and returns the evaluation "result",
 ;; 42.
-(test-assert-with-log "with-ssh"
+(test-equal-with-log "with-ssh"
+  42
   (run-client-test
    ;; server
    (lambda (server)
@@ -262,15 +266,16 @@ $4 = #<session #<undefined>@#<undefined>:22 (disconnected) 453fff>"
       (lambda (session)
         (format-log/scm 'nolog "client" "session: ~a" session)
         (let ((n (make-node session)))
-          (= (with-ssh n
-               (+ 21 21))
-             42)))))))
+          (with-ssh n
+            (+ 21 21))))))))
 
 ;;;
 
 
+(define exit-status (test-runner-fail-count (test-runner-current)))
+
 (test-end "dist")
 
-(exit (= (test-runner-fail-count (test-runner-current)) 0))
+(exit (= 0 exit-status))
 
 ;;; dist.scm ends here.
