@@ -48,7 +48,7 @@
 (define *sssh-cmd*
   (string-append
    %topbuilddir "/examples/sssh.scm"
-   " --identity-file=" %rsakey
+   " --identity-file=" %ecdsakey
    " --port=" (number->string *srv-port*)
    " --known-hosts-file=''"
    " " (inet-ntop AF_INET *srv-address*)
@@ -94,32 +94,36 @@
 
 ;;; Tests
 
-(test-assert "ssshd, start"
+(test-assert-with-log "ssshd, start"
   (let ((max-tries 10))
     (system *ssshd-cmd*)
     (let ((pid (wait-pid-file max-tries *srv-pid-file*)))
       (cleanup pid)
       pid)))
 
-(test-assert "sssh, exec"
+(test-assert-with-log "sssh, exec"
   (let ((max-tries 10))
+    (format (current-error-port) "test  command: ~A~%" *test-cmd*)
+    (format (current-error-port) "ssshd command: ~A~%" *ssshd-cmd*)
+    (format (current-error-port) "sssh  command: ~A~%" *sssh-cmd*)
     (system *ssshd-cmd*)
     (let* ((pid    (wait-pid-file max-tries *srv-pid-file*))
            (output (read-line (open-input-pipe *test-cmd*)))
            (p      (open-input-pipe *sssh-cmd*))
-           (result (let r ((res "")
-                           (l   (read-line p)))
-                     (and (not (eof-object? l))
-                          (if (string=? output res)
-                              #t
-                              (r (string-append res l)
-                                 (read-line p)))))))
+           (result (let ((line (read-line p)))
+                     (format (current-error-port) "      line: ~a~%" line)
+                     (and (not (eof-object? line))
+                          (string=? line output)))))
+      (format (current-error-port) "output: ~a~%" output)
+      (format (current-error-port) "result: ~a~%" result)
       (cleanup pid)
       result)))
 
 
+(define exit-status (test-runner-fail-count (test-runner-current)))
+
 (test-end "sssh-ssshd")
 
-(exit (= (test-runner-fail-count (test-runner-current)) 0))
+(exit (= 0 exit-status))
 
 ;;; sssh-ssshd.scm ends here.
